from unittest import TestCase
from uuid_shortener import UuidShortener
from uuid import uuid4, UUID


class TestUuidShortener(TestCase):

    def test_shorten_and_unshorten_with_prefix(self):
        uuid = uuid4()

        shortener = UuidShortener('dev')

        short_uuid = shortener.shorten(uuid)

        self.assertEqual(0, short_uuid.index('dev-'))
        self.assertEqual(uuid, shortener.unshorten(short_uuid))

    def test_shorten_and_unshorten_without_prefix(self):
        uuid = uuid4()

        shortener = UuidShortener()

        short_uuid = shortener.shorten(uuid)

        self.assertEqual(uuid, shortener.unshorten(short_uuid))

    def test_regression_1_left_padding_with_zeros(self):
        expected_uuid = UUID("0c1dda28-48fa-46e1-97e2-18e1c6d0fb60")

        shortened_uuid = '2uMfE2CfWdX3vJcN3yTWgJ'

        shortener = UuidShortener()

        self.assertEqual(expected_uuid,  shortener.unshorten(shortened_uuid))



from typing import Optional
from uuid import UUID
from pybaseconv import Converter, BASE


class UuidShortener:

    @property
    def prefix(self):
        return self._prefix

    def __init__(self, prefix: Optional[str] = None):
        self._prefix = prefix

    def shorten(self, uuid: UUID) -> str:
        """
        shortens the the given uuid and appends it to
        the prefix if any was provided
        :param uuid: The uuid number to shorten
        :return: the shortened version
        """
        # stringify the uuid and lower all letters
        stringified_uuid = str(uuid).lower()

        # remove all dashes
        stringified_uuid = stringified_uuid.replace('-', '')

        # uuid is a hex number, we convert it to FLICKER BASE 58 base
        # which give encoding that is suitable for usage in urls
        converter = Converter(BASE.HEX, BASE.FLICKER_BASE_58)
        converted_uuid = converter.convert(stringified_uuid)

        return f'{self.prefix}-{converted_uuid}' if \
            self.prefix is not None else converted_uuid

    def unshorten(self, short_uuid: str) -> UUID:
        """
        Strips the prefix from the given number, if any,
        and unshortens the rest.
        The given short number must have been shortened by
        the shorten method to avoid any errors.
        :param short_uuid: the shortened number to convert back to UUID
        :return: the UUID rep of the shortened number
        """

        prefix_substr = f'{self.prefix}-'

        if short_uuid.startswith(prefix_substr):
            short_uuid = short_uuid.replace(prefix_substr, '')

        converter = Converter(BASE.FLICKER_BASE_58, BASE.HEX)
        uuid_number = converter.convert(short_uuid)

        if len(uuid_number) < 32:
            left_pad = '0' * (32 - len(uuid_number))
            uuid_number = left_pad + uuid_number

        return UUID(uuid_number)



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

REQUIRES = ['pybaseconv==0.*']

setup(
    name="uuid-shortener-py",
    version="1.0.0",
    author="Samira El Aabidi",
    url='https://github.com/El-Sam/uuid_shortener',
    author_email="sam.elaabidi@gmail.com",
    keywords=["Python", "UUID", "UUID Shortener", "Hex base", "Short URL"],
    description="Shorten a uuid into a URL friendly format.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=REQUIRES,
    packages=['uuid_shortener'],
    classifiers=(
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ),
)
